#ifndef __CHEBY__FIP_URV_REGS__H__
#define __CHEBY__FIP_URV_REGS__H__

#include "mbox_regs.h"
#define FIP_URV_REGS_SIZE 16384 /* 0x4000 = 16KB */

/* PLC processor control */
#define FIP_URV_REGS_PLC_CTRL 0x0UL
#define FIP_URV_REGS_PLC_CTRL_RSTN 0x1UL

/* Status bits from NanoFIP */
#define FIP_URV_REGS_FIP_STATUS 0x4UL
#define FIP_URV_REGS_FIP_STATUS_VAR1_RDY 0x1UL
#define FIP_URV_REGS_FIP_STATUS_VAR3_RDY 0x2UL

/* Set access to var 1 */
#define FIP_URV_REGS_FIP_VAR1 0x8UL
#define FIP_URV_REGS_FIP_VAR1_ACC 0x1UL

/* Set access to var 1 */
#define FIP_URV_REGS_FIP_VAR3 0xcUL
#define FIP_URV_REGS_FIP_VAR3_ACC 0x1UL

/* None */
#define FIP_URV_REGS_MAILBOXES 0x10UL
#define ADDR_MASK_FIP_URV_REGS_MAILBOXES 0x3ff0UL
#define FIP_URV_REGS_MAILBOXES_SIZE 16 /* 0x10 */

/* servmod lines for boards */
#define FIP_URV_REGS_SERVMOD 0x20UL
#define FIP_URV_REGS_SERVMOD_LINE_MASK 0xffUL
#define FIP_URV_REGS_SERVMOD_LINE_SHIFT 0

/* Servmod control lines */
#define FIP_URV_REGS_SERVMOD_CTRL 0x24UL
#define FIP_URV_REGS_SERVMOD_CTRL_DIR 0x1UL

/* Status lines for PSU */
#define FIP_URV_REGS_PSU_STAT 0x28UL
#define FIP_URV_REGS_PSU_STAT_PRES_MASK 0x3UL
#define FIP_URV_REGS_PSU_STAT_PRES_SHIFT 0

/* Control lines for PSU */
#define FIP_URV_REGS_PSU_CTRL 0x2cUL
#define FIP_URV_REGS_PSU_CTRL_PWR_CYC_REQ_MASK 0x3UL
#define FIP_URV_REGS_PSU_CTRL_PWR_CYC_REQ_SHIFT 0

/* led */
#define FIP_URV_REGS_LEDS 0x30UL
#define FIP_URV_REGS_LEDS_VAL_MASK 0xfUL
#define FIP_URV_REGS_LEDS_VAL_SHIFT 0

/* Restart the fip urv (write 0x8e51a610) */
#define FIP_URV_REGS_RESTART 0x34UL

/* None */
#define FIP_URV_REGS_BOARDS 0x40UL
#define FIP_URV_REGS_BOARDS_SIZE 4 /* 0x4 */

/* None */
#define FIP_URV_REGS_BOARDS_PINS 0x0UL

/* i2c interfaces for monitors */
#define FIP_URV_REGS_I2C 0x60UL
#define ADDR_MASK_FIP_URV_REGS_I2C 0x3fe0UL
#define FIP_URV_REGS_I2C_SIZE 32 /* 0x20 */

/* flash */
#define FIP_URV_REGS_SPI 0x80UL
#define ADDR_MASK_FIP_URV_REGS_SPI 0x3fe0UL
#define FIP_URV_REGS_SPI_SIZE 32 /* 0x20 */

/* NanoFIP internal memory/registers */
#define FIP_URV_REGS_FIP_REG 0x800UL
#define ADDR_MASK_FIP_URV_REGS_FIP_REG 0x3800UL
#define FIP_URV_REGS_FIP_REG_SIZE 2048 /* 0x800 = 2KB */

/* Memory of the PLC urv */
#define FIP_URV_REGS_PLC_MEM 0x2000UL
#define ADDR_MASK_FIP_URV_REGS_PLC_MEM 0x2000UL
#define FIP_URV_REGS_PLC_MEM_SIZE 8192 /* 0x2000 = 8KB */

struct fip_urv_regs {
  /* [0x0]: REG (rw) PLC processor control */
  uint32_t plc_ctrl;

  /* [0x4]: REG (ro) Status bits from NanoFIP */
  uint32_t fip_status;

  /* [0x8]: REG (rw) Set access to var 1 */
  uint32_t fip_var1;

  /* [0xc]: REG (rw) Set access to var 1 */
  uint32_t fip_var3;

  /* [0x10]: SUBMAP (no description) */
  struct mbox_regs mailboxes;

  /* padding to: 8 words */
  uint32_t __padding_0[1];

  /* [0x20]: REG (ro) servmod lines for boards */
  uint32_t servmod;

  /* [0x24]: REG (rw) Servmod control lines */
  uint32_t servmod_ctrl;

  /* [0x28]: REG (ro) Status lines for PSU */
  uint32_t psu_stat;

  /* [0x2c]: REG (rw) Control lines for PSU */
  uint32_t psu_ctrl;

  /* [0x30]: REG (rw) led */
  uint32_t leds;

  /* [0x34]: REG (wo) Restart the fip urv (write 0x8e51a610) */
  uint32_t restart;

  /* padding to: 16 words */
  uint32_t __padding_1[2];

  /* [0x40]: REPEAT (no description) */
  struct boards {
    /* [0x0]: REG (ro) (no description) */
    uint32_t pins;
  } boards[8];

  /* [0x60]: SUBMAP i2c interfaces for monitors */
  uint32_t i2c[8];

  /* [0x80]: SUBMAP flash */
  uint32_t spi[8];

  /* padding to: 512 words */
  uint32_t __padding_2[472];

  /* [0x800]: SUBMAP NanoFIP internal memory/registers */
  uint32_t fip_reg[512];

  /* padding to: 2048 words */
  uint32_t __padding_3[1024];

  /* [0x2000]: SUBMAP Memory of the PLC urv */
  uint32_t plc_mem[2048];
};

#endif /* __CHEBY__FIP_URV_REGS__H__ */
